"""
Optional dependency checking.
"""

__all__ = ["UnavailableObjectError", "UnavailableObject", "check_optional_object"]

import sys

from ._version import get_versions

__version__ = get_versions()["version"]
del get_versions


class UnavailableObjectError(Exception):
    pass


class UnavailableObject:
    """
    An unavailable object.

    Args:
        name (str): Name of the unavailable object.
        msg (str, optional): Error message.
        none_child (bool): Return None instead of throwing an exception
            for __getattr and __getitem__.
    """

    def __init__(self, name, msg=None, none_child=False):
        self.name = name
        self.msg = msg
        self.none_child = none_child
        self._orig_exc = sys.exc_info()[1]

    def raise_(self, *_, **__):
        __tracebackhide__ = True

        msg = f"{self.name} is unavailable."
        if self.msg is not None:
            msg = f"{msg}\n\n{self.msg}"

        raise UnavailableObjectError(msg) from self._orig_exc

    __call__ = raise_

    def __getattr__(self, name):
        if self.none_child:
            return None
        self.raise_()

    def __getitem__(self, name):
        if self.none_child:
            return None

        self.raise_()


def check_available(*objects):
    """
    Check the availability of an optional object.

    This is optional but can be used to have somthing fail early.
    """
    __tracebackhide__ = True

    for obj in objects:
        if isinstance(obj, UnavailableObject):
            obj.raise_()



from setuptools import find_packages, setup

import versioneer

with open("README.md", "r") as fp:
    LONG_DESCRIPTION = fp.read()

setup(
    name="unavailable-object",
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    author="Simon-Martin Schroeder",
    author_email="martin.schroeder@nerdluecht.de",
    description="Optional Object",
    long_description=LONG_DESCRIPTION,
    long_description_content_type="text/markdown",
    url="https://github.com/moi90/unavailable-object",
    packages=find_packages("src"),
    package_dir={"": "src"},
    python_requires=">=3.6",
    extras_require={
        "tests": [
            # Pytest
            "pytest",
            "flake8",
            "black",
        ]
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
    ],
)
